/**
 * <copyright>
 *
 * Copyright (c) 2010 modelversioning.org
 * All rights reserved. This program and the accompanying materials are
 * made available under the terms of the Eclipse Public License v1.0 which
 * accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * </copyright>
 */

package org.modelversioning.operations.detection.evaluation;

import java.io.IOException;
import java.util.Collections;
import java.util.List;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.modelversioning.core.impl.UUIDResourceFactoryImpl;
import org.modelversioning.ecoremutator.EcoreMutator;
import org.modelversioning.ecoremutator.IModelProvider;
import org.modelversioning.ecoremutator.mutations.ModelProvider;
import org.modelversioning.ecoremutator.mutations.impl.AddAnnotationMutation;
import org.modelversioning.ecoremutator.mutations.impl.AddObjectMutation;
import org.modelversioning.ecoremutator.mutations.impl.DeleteObjectMutation;
import org.modelversioning.ecoremutator.mutations.impl.MoveObjectMutation;
import org.modelversioning.ecoremutator.mutations.impl.UnsetFeatureMutation;
import org.modelversioning.ecoremutator.mutations.impl.UpdateFeatureMutation;
import org.modelversioning.ecoremutator.tracker.IMutationTracker;
import org.modelversioning.ecoremutator.tracker.impl.CSVMutationTracker;

/**
 * @author <a href="mailto:langer@big.tuwien.ac.at">Philip Langer</a>
 *
 */
public class IncrementalMutator extends TestCase implements IMutationTracker {
	
	/*
	 * Configuration values:
	 */
	private String inputUri = "models/change-size/initial.ecore";
	private String outputUri100 = "models/change-size/1/revised.ecore";
	private String outputUri200 = "models/change-size/2/revised.ecore";
	private String outputUri300 = "models/change-size/3/revised.ecore";
	private String outputUri400 = "models/change-size/4/revised.ecore";
	private String outputUri500 = "models/change-size/5/revised.ecore";
	private int mutationCount = 100;
	
	
	
	ResourceSet resourceSet = new ResourceSetImpl();
	EcoreMutator mutator = new EcoreMutator();
	CSVMutationTracker tracker = new CSVMutationTracker();

	public void testMutate() throws IOException {
		configureResourceSet();
		initializeMutator();
		
		
		// load input model
		Resource inputResource = loadResource(inputUri);
		Assert.assertTrue(inputResource.getContents()
				.size() > 0);
		
		// initialize model provider
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(inputResource);
		
		// mutate model
		mutator.mutate(modelProvider, mutationCount);
		
		// save output model 100
		Resource outputResource100 = createResource(outputUri100);
		outputResource100.getContents().addAll(inputResource.getContents());
		outputResource100.save(Collections.emptyMap());
		
		// re-initialize model provider
		modelProvider = new ModelProvider();
		modelProvider.setModelResource(outputResource100);
		
		// mutate model
		mutator.mutate(modelProvider, mutationCount);
		
		// save output model 200
		Resource outputResource200 = createResource(outputUri200);
		outputResource200.getContents().addAll(outputResource100.getContents());
		outputResource200.save(Collections.emptyMap());
		
		// re-initialize model provider
		modelProvider = new ModelProvider();
		modelProvider.setModelResource(outputResource200);
		
		// mutate model
		mutator.mutate(modelProvider, mutationCount);
		
		// save output model 300
		Resource outputResource300 = createResource(outputUri300);
		outputResource300.getContents().addAll(outputResource200.getContents());
		outputResource300.save(Collections.emptyMap());
		
		// re-initialize model provider
		modelProvider = new ModelProvider();
		modelProvider.setModelResource(outputResource300);
		
		// mutate model
		mutator.mutate(modelProvider, mutationCount);
		
		// save output model 300
		Resource outputResource400 = createResource(outputUri400);
		outputResource400.getContents().addAll(outputResource300.getContents());
		outputResource400.save(Collections.emptyMap());
		
		// re-initialize model provider
		modelProvider = new ModelProvider();
		modelProvider.setModelResource(outputResource400);
		
		// mutate model
		mutator.mutate(modelProvider, mutationCount);
		
		// save output model 300
		Resource outputResource500 = createResource(outputUri500);
		outputResource500.getContents().addAll(outputResource400.getContents());
		outputResource500.save(Collections.emptyMap());
		
	}


	private void initializeMutator() {
		mutator.addMutation(new AddAnnotationMutation());
		mutator.addMutation(new AddObjectMutation());
		mutator.addMutation(new DeleteObjectMutation());
		mutator.addMutation(new MoveObjectMutation());
		mutator.addMutation(new UnsetFeatureMutation());
		mutator.addMutation(new UpdateFeatureMutation());
		// TODO fix tracker and change it to tracker instance
		mutator.setTracker(this);
	}

	private void configureResourceSet() {
		// init resource set
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("ecore", new UUIDResourceFactoryImpl());
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("xmi", new UUIDResourceFactoryImpl());
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("operation", new UUIDResourceFactoryImpl());
	}
	
	
	/**
	 * Loads the resource with the specified <code>fileURI</code>.
	 * 
	 * @param fileURI
	 *            to load.
	 * @return loaded resource.
	 */
	private Resource loadResource(String fileURIString) {
		URI fileURI = URI.createURI(fileURIString, true);
		return resourceSet.getResource(fileURI, true);
	}
	
	/**
	 * Creates the resource with the specified <code>fileURI</code>.
	 * 
	 * @param fileURI
	 *            to load.
	 * @return loaded resource.
	 */
	private Resource createResource(String fileURIString) {
		URI fileURI = URI.createURI(fileURIString, true);
		return resourceSet.createResource(fileURI);
	}


	@Override
	public void track(String mutatorId, String message, boolean successful,
			List<EObject> involvedObjects,
			List<EStructuralFeature> involvedFeatures) {
		// TODO Auto-generated method stub
		
	}

}
