/**
 * <copyright>
 *
 * Copyright (c) 2010 modelversioning.org
 * All rights reserved. This program and the accompanying materials are
 * made available under the terms of the Eclipse Public License v1.0 which
 * accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * </copyright>
 */

package org.modelversioning.operations.detection.evaluation;

import java.io.IOException;
import java.util.Collections;
import java.util.List;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.modelversioning.core.impl.UUIDResourceFactoryImpl;
import org.modelversioning.ecoremutator.EcoreMutator;
import org.modelversioning.ecoremutator.IModelProvider;
import org.modelversioning.ecoremutator.mutations.ModelProvider;
import org.modelversioning.ecoremutator.mutations.impl.AddAnnotationMutation;
import org.modelversioning.ecoremutator.mutations.impl.AddObjectMutation;
import org.modelversioning.ecoremutator.mutations.impl.DeleteObjectMutation;
import org.modelversioning.ecoremutator.mutations.impl.MoveObjectMutation;
import org.modelversioning.ecoremutator.mutations.impl.UnsetFeatureMutation;
import org.modelversioning.ecoremutator.mutations.impl.UpdateFeatureMutation;
import org.modelversioning.ecoremutator.tracker.IMutationTracker;
import org.modelversioning.ecoremutator.tracker.impl.CSVMutationTracker;

/**
 * @author <a href="mailto:langer@big.tuwien.ac.at">Philip Langer</a>
 *
 */
public class Mutate extends TestCase implements IMutationTracker {
	
	/*
	 * Configuration values:
	 */
	private String inputUri = "models/model-size/200/initial.ecore";
	private String outputUri = "models/model-size/200/revised.ecore";
	private String csvOutputFile = "";
	private int mutationCount = 50;
	
	
	
	ResourceSet resourceSet = new ResourceSetImpl();
	EcoreMutator mutator = new EcoreMutator();
	CSVMutationTracker tracker = new CSVMutationTracker();

	public void testMutate() throws IOException {
		configureResourceSet();
		initializeMutator();
		
		
		// load input model
		Resource inputResource = loadResource(inputUri);
		Assert.assertTrue(inputResource.getContents()
				.size() > 0);
		
		// initialize model provider
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(inputResource);
		
		// mutate model
		mutator.mutate(modelProvider, mutationCount);
		
		// save output model
		Resource outputResource = createResource(outputUri);
		outputResource.getContents().addAll(inputResource.getContents());
		outputResource.save(Collections.emptyMap());
		
		// TODO save tracker output
		
	}


	private void initializeMutator() {
		mutator.addMutation(new AddAnnotationMutation());
		mutator.addMutation(new AddObjectMutation());
		mutator.addMutation(new DeleteObjectMutation());
		mutator.addMutation(new MoveObjectMutation());
		mutator.addMutation(new UnsetFeatureMutation());
		mutator.addMutation(new UpdateFeatureMutation());
		// TODO fix tracker and change it to tracker instance
		mutator.setTracker(this);
	}

	private void configureResourceSet() {
		// init resource set
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("ecore", new UUIDResourceFactoryImpl());
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("xmi", new UUIDResourceFactoryImpl());
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("operation", new UUIDResourceFactoryImpl());
	}
	
	
	/**
	 * Loads the resource with the specified <code>fileURI</code>.
	 * 
	 * @param fileURI
	 *            to load.
	 * @return loaded resource.
	 */
	private Resource loadResource(String fileURIString) {
		URI fileURI = URI.createURI(fileURIString, true);
		return resourceSet.getResource(fileURI, true);
	}
	
	/**
	 * Creates the resource with the specified <code>fileURI</code>.
	 * 
	 * @param fileURI
	 *            to load.
	 * @return loaded resource.
	 */
	private Resource createResource(String fileURIString) {
		URI fileURI = URI.createURI(fileURIString, true);
		return resourceSet.createResource(fileURI);
	}


	@Override
	public void track(String mutatorId, String message, boolean successful,
			List<EObject> involvedObjects,
			List<EStructuralFeature> involvedFeatures) {
		// TODO Auto-generated method stub
		
	}

}
